local metadata = {
"## Interface: FS17 1.5.3.1 b1847",
"## Title:mixerWagonFoodSwitch",
"## Notes: Specialization for MixerWagon",
"## Author: Marhu",
"## Version: 1.0.0-44",
"## Date: 27.06.2018",
"## Web: http://marhu.net"
}

--[[
	Specialization für MixerWagon um anhand der ersten eingefüllten Zutat
	das zu erstellende Mischfutter zu wählen.
]]
 
mixerWagonFoodSwitch = {};

function mixerWagonFoodSwitch.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations) and not SpecializationUtil.hasSpecialization(MixerWagon, specializations);
end

function mixerWagonFoodSwitch:load(savegame)
	self.setUnitFillLevel = Utils.overwrittenFunction(self.setUnitFillLevel, mixerWagonFoodSwitch.setUnitFillLevel);
	
end

function mixerWagonFoodSwitch:postLoad(savegame)

	if not SpecializationUtil.hasSpecialization(MixerWagon, self.specializations) then
		return BaseMission.VEHICLE_LOAD_ERROR;
	end
	
	self.allowFillType = Utils.overwrittenFunction(self.allowFillType, mixerWagonFoodSwitch.allowFillType);
	
	local uiScale = g_gameSettings:getValue("uiScale")

	self.mixType = FillUtil.FILLTYPE_FORAGE;
	local mixType = getXMLString(self.xmlFile, "vehicle.mixerWagonFillTypes#mixType");
    if mixType ~= nil then
		local mixTypInt = FillUtil.fillTypeNameToInt[mixType];
        if mixTypInt ~= nil then
			self.mixType = mixTypInt;
		else
			print("Warning: InvalidmixerWagonFoodSwitchMixType '"..mixType.."' in '"..self.configFileName.. "'.");
		end
	end
    
	
	self.currentFood = 1;
	self.mixerWagonFillTypesFood = {};
    self.fillTypeToMixerWagonFillTypeFood = {};
	self.mixerWagonFillTypesFood[1] = {};
	self.mixerWagonFillTypesFood[1].food = self.mixType;
	self.mixerWagonFillTypesFood[1].types = self.mixerWagonFillTypes;
	self.fillTypeToMixerWagonFillTypeFood[1] = self.fillTypeToMixerWagonFillType;
	
	local i=2;
	while true do
		local baseName = string.format("vehicle.mixerWagonFillTypes%d", i);
		if not hasXMLProperty(self.xmlFile, baseName) then
			break;
		end
		
		self.mixerWagonFillTypesFood[i] = {};
		self.mixerWagonFillTypesFood[i].types ={};
		self.fillTypeToMixerWagonFillTypeFood[i] = {};
				
		self.mixerWagonFillTypesFood[i].food = FillUtil.FILLTYPE_FORAGE;
		local mixType = getXMLString(self.xmlFile,  baseName.."#mixType");
		if mixType ~= nil then
			local mixTypInt = FillUtil.fillTypeNameToInt[mixType];
			if mixTypInt ~= nil then
				self.mixerWagonFillTypesFood[i].food = mixTypInt;
			else
				print("Warning: InvalidmixerWagonFoodSwitchMixType '"..mixType.."' in '"..self.configFileName.. "'.");
			end
		end
		
		local j=0;
		while true do
			local baseName = baseName..string.format(".mixerWagonFillType(%d)", j);
			if not hasXMLProperty(self.xmlFile, baseName) then
				break;
			end
			local entry = {};
			entry.fillTypes = {};

			local k=0;
			while true do
				local fillTypeKey = baseName..string.format(".fillType(%d)", k);
				if not hasXMLProperty(self.xmlFile, fillTypeKey) then
					break;
				end
				local fillType = getXMLString(self.xmlFile, fillTypeKey.."#fillType");
				if fillType ~= nil then
					local fillTypeInt = FillUtil.fillTypeNameToInt[fillType];
					if fillTypeInt ~= nil then
						if self.fillTypeToMixerWagonFillTypeFood[i][fillTypeInt] == nil then
							entry.fillTypes[fillTypeInt] = true;
							self.fillTypeToMixerWagonFillTypeFood[i][fillTypeInt] = entry;
						else
							print("Warning: Invalid mixerWagonFoodSwitchFillType '"..fillType.."' used multiple times in '"..self.configFileName.."'.");
						end
					else
						print("Warning: Invalid mixerWagonFoodSwitchFillType '"..fillType.."' in '"..self.configFileName.. "'.");
					end
				end

				k = k+1;
			end
			entry.minPercentage = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.."#minPercentage"), 0)*0.01;
			entry.maxPercentage = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.."#maxPercentage"), 100)*0.01;
			entry.fillLevel = 0;

			local name = Utils.getNoNil(getXMLString(self.xmlFile, baseName.."#name"), "unknown");
			entry.name = name;
			local i18n = g_i18n;
			if self.customEnvironment ~= nil then
				i18n = _G[self.customEnvironment].g_i18n;
			end
			entry.nameI18N = i18n:getText("fillType_" .. name);

			local fillTypeDesc = FillUtil.fillTypeNameToDesc[name];
			if fillTypeDesc ~= nil then
				local width, height = getNormalizedScreenValues(20*uiScale, 20*uiScale);
				entry.overlay = Overlay:new("MixerWagonFillType", fillTypeDesc.hudOverlayFilenameSmall, 0, 0, width, height)
				entry.overlay:setColor(unpack(g_currentMission.helpBoxTextColor))
			end

			if next(entry.fillTypes) ~= nil then
				entry.statusBarColor = {0.2122, 0.5271, 0.0307, 1};
				entry.statusBarColor2 = {0.2832, 0.0091, 0.0091, 1}
				local width, height = getNormalizedScreenValues(208*uiScale, 8*uiScale);
				entry.statusBar = StatusBar:new(g_baseUIFilename, g_colorBgUVs, nil, {0.0075, 0.0075, 0.0075, 1}, entry.statusBarColor2, nil, 0, 0, width, height);
				table.insert(self.mixerWagonFillTypesFood[i].types, entry);
			end
			j = j+1;
		end
		i = i+1;
    end
	
    if savegame ~= nil then
       		
		local foodTypeKey = savegame.key..".mixerWagonFoodType#name";
		local foodType = getXMLString(savegame.xmlFile, foodTypeKey);
		local foodTypeInt = FillUtil.fillTypeNameToInt[foodType];
		if foodTypeInt ~= nil and foodTypeInt ~= self.mixType then
			self:setUnitFillLevel(self.mixerWagon.fillUnitIndex, 0, FillUtil.FILLTYPE_UNKNOWN, true);
			for i=1, table.getn(self.mixerWagonFillTypesFood) do
				if self.mixerWagonFillTypesFood[i].food == foodTypeInt then
					self.mixType = self.mixerWagonFillTypesFood[i].food;
					self.mixerWagonFillTypes = self.mixerWagonFillTypesFood[i].types;
					self.fillTypeToMixerWagonFillType = self.fillTypeToMixerWagonFillTypeFood[i];
					self.currentFood = i;
					break;
				end
			end	
			for i=1, table.getn(self.mixerWagonFillTypes) do
				local fillTypeKey = savegame.key..string.format(".mixerWagonFillType%d#fillLevel", i);
				local fillLevel = getXMLFloat(savegame.xmlFile, fillTypeKey);
				if fillLevel == nil then
					fillLevel = 0;
				end
				if fillLevel > 0 then
					fillLevel = fillLevel + self:getFillLevel( next(self.mixerWagonFillTypes[i].fillTypes) );
					self:setUnitFillLevel(self.mixerWagon.fillUnitIndex, fillLevel, next(self.mixerWagonFillTypes[i].fillTypes));
				end
			end
		end
    end
	return BaseMission.VEHICLE_LOAD_OK;
end

function mixerWagonFoodSwitch:delete()
end
function mixerWagonFoodSwitch:getSaveAttributesAndNodes(nodeIdent)
	local fillType = FillUtil.fillTypeIntToName[self.mixType];
    return "", nodeIdent..'<mixerWagonFoodType name="'..fillType..'" />';
end
function mixerWagonFoodSwitch:readStream(streamId, connection)
	local currentFood = streamReadInt8(streamId)
	if self.currentFood ~= currentFood then
		self.mixType = self.mixerWagonFillTypesFood[currentFood].food;
		self.mixerWagonFillTypes = self.mixerWagonFillTypesFood[currentFood].types;
		self.fillTypeToMixerWagonFillType = self.fillTypeToMixerWagonFillTypeFood[currentFood];
		self.currentFood = currentFood;
	end
end
function mixerWagonFoodSwitch:writeStream(streamId, connection)
	streamWriteInt8(streamId, self.currentFood)
end
function mixerWagonFoodSwitch:readUpdateStream(streamId, timestamp, connection)
	if connection:getIsServer() then
		local currentFood = streamReadInt8(streamId)
		if self.currentFood ~= currentFood then
			self:setUnitFillLevel(self.mixerWagon.fillUnitIndex, 0, FillUtil.FILLTYPE_UNKNOWN, true);
			self.mixType = self.mixerWagonFillTypesFood[currentFood].food;
			self.mixerWagonFillTypes = self.mixerWagonFillTypesFood[currentFood].types;
			self.fillTypeToMixerWagonFillType = self.fillTypeToMixerWagonFillTypeFood[currentFood];
			self.currentFood = currentFood;
		end
	end
end
function mixerWagonFoodSwitch:writeUpdateStream(streamId, connection, dirtyMask)
	if not connection:getIsServer() then
		streamWriteInt8(streamId, self.currentFood)
	end
end
function mixerWagonFoodSwitch:mouseEvent(posX, posY, isDown, isUp, button)
end
function mixerWagonFoodSwitch:keyEvent(unicode, sym, modifier, isDown)
end
function mixerWagonFoodSwitch:update(dt)
end
function mixerWagonFoodSwitch:updateTick(dt)
end
function mixerWagonFoodSwitch:draw()
end

function mixerWagonFoodSwitch:allowFillType(superFunc, fillType, allowEmptying)
	local fillLevel = self:getFillLevel();
	if fillLevel <= (self:getUnitCapacity(self.mixerWagon.fillUnitIndex) * self.fillTypeChangeThreshold) then
		local mixerWagonFillType = self.fillTypeToMixerWagonFillType[fillType];
		if mixerWagonFillType == nil then
			for i=1, table.getn(self.fillTypeToMixerWagonFillTypeFood) do
				local mixerWagonFillTypeNew = self.fillTypeToMixerWagonFillTypeFood[i][fillType];
				if mixerWagonFillTypeNew ~= nil then
					self.mixType = self.mixerWagonFillTypesFood[i].food;
					self.mixerWagonFillTypes = self.mixerWagonFillTypesFood[i].types;
					self.fillTypeToMixerWagonFillType = self.fillTypeToMixerWagonFillTypeFood[i];
					self.currentFood = i;
					break;
				end
			end
		end
	end
    return superFunc(self, fillType, allowEmptying);
end

function mixerWagonFoodSwitch:setUnitFillLevel(superFunc, fillUnitIndex, fillLevel, fillType, force, fillInfo)
	if fillType == FillUtil.FILLTYPE_FORAGE and self.mixType ~= nil then
		fillType = self.mixType;
	end
	superFunc(self, fillUnitIndex, fillLevel, fillType, force, fillInfo);
end

